local RW = LibStub("AceAddon-3.0"):GetAddon("Raid Watch")
if not RW then return end
local L = LibStub("AceLocale-3.0"):GetLocale("RW-2.0")

local Party

RW.defaults = {
	profile = {
		Font = "accid",
		FontSize = 16,
		FontOutline = "OUTLINE",
		FontColor = {0.47, 0.75, 1, 1},
		FontAlign = "LEFT",
		BGTexture = "Solid",
		BGColor = {0, 0, 0, 0.5},
		Border = "IshBorder",
		BorderColor = {0.15, 0.24, 0.31, 1},
		BorderSize = 12,
		BarTexture = "HalD",
		BarColor = {0.22, 0.34, 0.46, 1},
		Insets = 1,
		BorderMargin = 5
	}
}

function RW:InTable(t, entry)
	for i, v in pairs(t) do
		if v == entry then return true end
	end
end

function RW:InTable2(t, key, entry)
	for i, v in pairs(t) do
		if v[key] == entry then return true end
	end
end

function RW:GetTableData(t, key, entry)
	for i, v in pairs(t) do
		if v[key] == entry then return v end
	end
end

function RW:GetIndexForvalue(t, value)
	for i, v in pairs(t) do
		if v == value then return i end
	end
end

function RW:NrElements(t)
	local count = 0
	for i, v in pairs(t) do
		count = count + 1
	end
	return count
end

function RW:TRemove(t, entry)
	for i, v in pairs(t) do
		if v == entry then 
			tremove(t, i)
		end
	end
end

function RW:CopyTable(t, d)
	if (d  or 0) > 5 then return nil end
	local new = {}
	for i, v in pairs(t) do
		if type(v) == "table" then
			new[i] = self:CopyTable(v, (d or 0) + 1)
		else
			new[i] = v
		end
	end
	return new
end

function RW:TimeToStr(time)
	if type(time) ~= "number" then return time end
	time = math.floor(time)
	local t
	if time < 60 then
		return L.COMBATTIME_SEC:format(time)
	elseif time % 60 == 0 then
		return L.COMBATTIME_MIN:format(time/60)
	else
		return L.COMBATTIME:format(time/60, time % 60)
	end
end

function RW:TimeToStringShort(t)
	if type(t) ~= "number" then return t end
	if t <= 10 then
		return ("%.1fs"):format(t)
	elseif t <= 60 then
		return ("%ds"):format(t)
	else
		return ("%0.2d:%0.2d"):format(t/60, math.fmod(t, 60))
	end
end

function RW:RGBConvert(r, g, b, a)
	return r/255, g/255, b/255, (a and a/255 or 1)
end

function RW:Lerp(s, e, a)
	if not s or not e or not a then return 0 end
	return s + ((e - s) * a)
end

function RW:AddBit(t, b)
	if bit.band(t or 0, b or 0) ~= b then t = (t or 0) + (b or 0) end
	return (t or 0)
end

function RW:RemoveBit(t, b)
	if bit.band(t or 0, b or 0) == b then t = (t or 0) - (b or 0) end
	return (t or 0)
end

function RW:ColorNames(str)
	if not Party then Party = RW:GetPlugin("Party") end
	str = str:gsub(">([%S\s-]+)<", function(name)
		if Party.group[name] then
			local color = RAID_CLASS_COLORS[Party.group[name].class]
			if color then
				name = ("|cff%.2x%.2x%.2x%s|r"):format(color.r * 255, color.g * 255, color.b * 255, name)
			end
		end
		return name
	end)
	return str
end

-- A bit of hack to solve if the first argument is correct but the rest is not, 
-- only for timer and message formating realy.
function RW:Format(text, ...)
	local ok, str = pcall(format, text, ...)
	if not ok then
		local first = select(1, ...)
		ok, str = pcall(format, text, first, L.UNKNOWN, 0)
	end
	if not ok then
		local first = select(1, ...)
		ok, str = pcall(format, text, first, 0)
	end
	return ok and str or text:gsub("(%%.)", L.UNKNOWN)
end

do
	local playerSpells
	function RW:GetRandomSpell()
		if not playerSpells then
			playerSpells = {}
			local i = 1
			while true do
				local spellName = GetSpellName(i, BOOKTYPE_SPELL)
				if not spellName then break end
				playerSpells[#playerSpells + 1] = spellName
				i = i + 1
			end
		end
		local name, rank, icon = GetSpellInfo(playerSpells[math.random(1, #playerSpells)])
		return name, rank, icon
	end
end

-----------------------------
-- Party functions
function RW:IsHeroic()
	local name, instanceType, difficultyIndex, difficultyName, maxPlayers, playerDifficulty, isDynamic = GetInstanceInfo()
	if (instanceType == "party" and difficultyIndex == 2) or difficultyIndex > 2 then return true end
	if instanceType == "none" then return false end
end

function RW:InstanceSize()
	local _, instanceType, difficulty, _, maxPlayers = GetInstanceInfo()
	if instanceType == "none" then return 10 end
	return maxPlayers
end

function RW:CreateAnchor(name, parent, text)
	local frame = CreateFrame("Frame", name or nil, parent or nil)
	frame:SetPoint("CENTER")
	frame:SetWidth(200)
	frame:SetHeight(25)
	frame:EnableMouse(true)
	frame:RegisterForDrag("LeftButton")
	frame:SetMovable(true)

	local bg = frame:CreateTexture(nil, "PARENT")
	bg:SetAllPoints(frame)
	bg:SetTexture(0.39, 0.55, 0.87, 0.7)
	frame.background = bg
	
	local str = frame:CreateFontString(name and name.."Text", "ARTWORK")
	str:SetFont(LibStub("LibSharedMedia-3.0"):Fetch("font", "accid"), 16, "OUTLINE")
	str:SetAllPoints()
	str:SetText(text)
	
	frame.str = str
	frame:Hide()
	return frame
end

function RW:CreateBar(rightClickFunc)
	local db = self.defaults.profile
	local frame = CreateFrame("Frame", "RW_Bar", UIParent)
	local bar = CreateFrame("StatusBar", frame:GetName().."Bar", frame)
	local timeText = bar:CreateFontString(frame:GetName().."TimeText", "ARTWORK")
	local text = bar:CreateFontString(frame:GetName().."Text", "ARTOWRK")
	local icon = frame:CreateTexture(frame:GetName().."Icon", "ARTWORK")
	
	frame:SetPoint("CENTER")
	frame:SetWidth(255)
	bar:SetPoint("BOTTOMRIGHT", frame, "BOTTOMRIGHT", -db.BorderMargin, db.BorderMargin)
	bar:SetPoint("TOPRIGHT", frame, "TOPRIGHT", -db.BorderMargin, -db.BorderMargin)
	icon:SetPoint("TOPLEFT", frame, "TOPLEFT", db.BorderMargin, -db.BorderMargin)
	icon:SetPoint("BOTTOMRIGHT", bar, "BOTTOMLEFT", -2, 0)
	timeText:SetPoint("RIGHT", bar, "RIGHT", -4, 0)
	text:SetPoint("LEFT", bar, "LEFT", 2, 0)
	text:SetPoint("RIGHT", timeText, "LEFT")
	
	bar:SetMinMaxValues(0,1)
	
	timeText:SetJustifyH("LEFT")
	timeText:SetWidth(45)
	
	frame.timeText = timeText
	frame.text = text
	frame.bar = bar
	frame.icon = icon
	
	frame.rightClickfuc = rightClickFunc
	
	frame:SetScript("OnEvent", function(self, event, arg1, arg2)
		if arg1 == "LSHIFT" or arg1 == "RSHIFT" then
			if arg2 == 1 then
				frame:EnableMouse(true)
				self:SetScript("OnMouseDown", self.rightClickfuc)
			else
				frame:EnableMouse(false)
				self:SetScript("OnMouseDown", nil)
			end
		end
	end)
	frame:Hide()
	return frame
end

---------------------
-- AceGUI Helpers
function RW.SetGUIOption(widget, event, ...)
	local db = widget:GetUserData("db")
	local key = widget:GetUserData("key")
	
	if select("#", ...) > 1 then
		for i = 1, select("#", ...) do
			db[key][i] = select(i, ...)
		end
	else
		db[key] = ...
	end
end

function RW:PrepareWidget(widget, db, key)
	widget:SetUserData("db", db)
	widget:SetUserData("key", key)
	widget:SetCallback("OnValueChanged", RW.SetGUIOption)
end
